@@ -152,6 +152,15 @@ class ProteinManager:
         return self.datastore.get_ID_type(ID)
 
     def get_isoforms(self, ID):
+        """ Returns a dictionary, where the keys are isoform names
+            and the values are the full isoform sequences associated
+            with those values as reconstructed from NCBI records.
+
+            If NCBI lookup fails and we fall back to UniProt, then
+            The UniProt isoform sequences will be queried directly,
+            but this takes longer as it requires additional network
+            requests"
+        """
         return self.datastore.get_isoforms(ID)
 
     def run_translation(self, Acc):
